/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.cluster.jgroups;

import java.io.Serializable;
import java.util.Vector;

import org.jgroups.Address;
import org.jgroups.Channel;

/**
 * An interface of the JGroups distributed tree.
 * @author Loris Bouzonnet
 */
public interface IDistributedTree {

    void start() throws Exception;

    void start(long timeout) throws Exception;

    void stop();

    void add(String fqn);

    void add(String fqn, Serializable element);

    /** resets an existing node, useful after a merge when you want to tell other
     *  members of your state, but do not wish to remove and then add as two separate calls */
    void reset(String fqn, Serializable element);

    void remove(String fqn);

    void add(String fqn, Serializable element, int timeout);

    /** resets an existing node, useful after a merge when you want to tell other
     *  members of your state, but do not wish to remove and then add as two separate calls */
    void reset(String fqn, Serializable element, int timeout);

    void remove(String fqn, int timeout);

    boolean exists(String fqn);

    Serializable get(String fqn);

    void set(String fqn, Serializable element);

    void set(String fqn, Serializable element, int timeout);

    void addDistributedTreeListener(
            DistributedTreeListener distributedTreeListener);

    void removeDistributedTreeListener(DistributedTreeListener distributedTreeListener);

    void addViewListener(ViewListener listener);

    void removeViewListener(ViewListener listener);

    Channel getChannel();

    Address getLocalAddress();

    Vector<String> getChildrenNames(String fqn);

    interface DistributedTreeListener {
        void nodeAdded(String fqn, Serializable element);

        void nodeRemoved(String fqn);

        void nodeModified(String fqn, Serializable old_element, Serializable new_element);
    }

    interface ViewListener {
        void viewChange(Vector<Address> new_mbrs, Vector<Address> old_mbrs);
    }

}
