/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.cluster.jgroups;

import java.util.Vector;

import org.jgroups.Channel;
import org.jgroups.JChannel;
import org.jgroups.MembershipListener;
import org.jgroups.Message;
import org.jgroups.MessageListener;
import org.jgroups.SuspectedException;
import org.jgroups.TimeoutException;
import org.jgroups.blocks.MessageDispatcher;
import org.jgroups.blocks.RequestHandler;
import org.jgroups.util.RspList;

/**
 * Wrap an instance of {@link MessageDispatcher}.
 * @author Loris Bouzonnet
 */
public class MessageDispatcherWrapper implements IMessageDispatcher {

    private final MessageDispatcher messageDispatcher;

    public MessageDispatcherWrapper(final MessageDispatcher messageDispatcher) {
        super();
        this.messageDispatcher = messageDispatcher;
    }

    public MessageDispatcherWrapper(
            final JChannel channel, final MessageListener messageListener, final MembershipListener membershipListener, final RequestHandler requestHandler) {
        messageDispatcher = new MessageDispatcher(channel, messageListener, membershipListener, requestHandler);
    }

    public RspList castMessage(final Vector dests, final Message msg, final int mode, final long timeout) {
        return messageDispatcher.castMessage(dests, msg, mode, timeout);
    }

    public Object sendMessage(final Message msg, final int mode, final long timeout)
    throws TimeoutException, SuspectedException {
        return messageDispatcher.sendMessage(msg, mode, timeout);
    }

    public Channel getChannel() {
        return messageDispatcher.getChannel();
    }

    public void stop() {
        messageDispatcher.stop();
    }

}
